@@ -209,11 +209,10 @@ namespace Banshee.Dap.AppleDevice
                 });
             }
 
-            var invalid_tracks = new List<GPod.Track> ();
             foreach (var ipod_track in MediaDatabase.Tracks) {
 
                 if (String.IsNullOrEmpty (ipod_track.IpodPath)) {
-                    invalid_tracks.Add (ipod_track);
+                    invalid_tracks_in_device.Enqueue (ipod_track);
                     continue;
                 }
 
@@ -228,13 +227,10 @@ namespace Banshee.Dap.AppleDevice
                     Log.Exception (e);
                 }
             }
-            if (invalid_tracks.Count > 0) {
-                Log.Warning (String.Format ("Found {0} invalid tracks on the device", invalid_tracks.Count));
-                foreach (var track in invalid_tracks) {
-                    DeleteTrack (track, false);
-                }
-            }
 
+            if (invalid_tracks_in_device.Count > 0) {
+                Log.Warning (String.Format ("Found {0} invalid tracks on the device", invalid_tracks_in_device.Count));
+            }
 
             Hyena.Data.Sqlite.HyenaSqliteCommand insert_cmd = new Hyena.Data.Sqlite.HyenaSqliteCommand (
                 @"INSERT INTO CorePlaylistEntries (PlaylistID, TrackID)
@@ -385,6 +381,7 @@ namespace Banshee.Dap.AppleDevice
         private Queue<AppleDeviceTrackInfo> tracks_to_add = new Queue<AppleDeviceTrackInfo> ();
         private Queue<AppleDeviceTrackInfo> tracks_to_update = new Queue<AppleDeviceTrackInfo> ();
         private Queue<AppleDeviceTrackInfo> tracks_to_remove = new Queue<AppleDeviceTrackInfo> ();
+        private Queue<GPod.Track> invalid_tracks_in_device = new Queue<GPod.Track> ();
 
         private uint sync_timeout_id = 0;
         private object sync_timeout_mutex = new object ();
@@ -566,14 +563,32 @@ namespace Banshee.Dap.AppleDevice
         {
             Hyena.Log.Debug ("Starting AppleDevice sync thread cycle");
 
-            string message;
-            int total, i = 0;
             var progressUpdater = new UserJob (Catalog.GetString ("Syncing iPod"),
                                                Catalog.GetString ("Preparing to synchronize..."), GetIconNames ());
             progressUpdater.Register ();
             MediaDatabase.StartSync ();
-            message = Catalog.GetString ("Adding track {0} of {1}");
-            total = tracks_to_add.Count;
+
+            SyncTracksToAdd (progressUpdater);
+
+            SyncTracksToUpdate ();
+
+            SyncTracksToRemove (progressUpdater);
+
+            SyncTracksToPlaylists ();
+
+            SyncDatabase (progressUpdater);
+
+            MediaDatabase.StopSync ();
+            progressUpdater.Finish ();
+
+            Hyena.Log.Debug ("Ending AppleDevice sync thread cycle");
+        }
+
+        void SyncTracksToAdd (UserJob progressUpdater)
+        {
+            string message = Catalog.GetString ("Adding track {0} of {1}");
+            int total = tracks_to_add.Count;
+            int i = 0;
             while (tracks_to_add.Count > 0) {
                 AppleDeviceTrackInfo track = null;
                 lock (sync_mutex) {
@@ -594,7 +609,10 @@ namespace Banshee.Dap.AppleDevice
                 OnTracksAdded ();
                 OnUserNotifyUpdated ();
             }
+        }
 
+        void SyncTracksToUpdate ()
+        {
             while (tracks_to_update.Count > 0) {
                 AppleDeviceTrackInfo track = null;
                 lock (sync_mutex) {
@@ -607,9 +625,12 @@ namespace Banshee.Dap.AppleDevice
                     Log.Exception ("Cannot save track to iPod", e);
                 }
             }
+        }
 
-            message = Catalog.GetString ("Removing track {0} of {1}");
-            total = tracks_to_remove.Count;
+        void SyncTracksToRemove (UserJob progressUpdater)
+        {
+            string message = Catalog.GetString ("Removing track {0} of {1}");
+            int total = tracks_to_remove.Count;
             while (tracks_to_remove.Count > 0) {
                 AppleDeviceTrackInfo track = null;
                 lock (sync_mutex) {
@@ -633,35 +654,56 @@ namespace Banshee.Dap.AppleDevice
                 }
             }
 
-            if (SupportsPlaylists) {
-                // Remove playlists on the device
-                var device_playlists = new List<GPod.Playlist> (MediaDatabase.Playlists);
-                foreach (var playlist in device_playlists) {
-                    if (!playlist.IsMaster && !playlist.IsPodcast) {
-                        MediaDatabase.Playlists.Remove (playlist);
-                    }
-                }
+            SyncRemovalOfInvalidTracks ();
+        }
 
-                // Add playlists from Banshee to the device
-                foreach (Source child in Children) {
-                    PlaylistSource from = child as PlaylistSource;
-                    if (from != null && from.Count > 0) {
-                        var playlist = new GPod.Playlist (from.Name);
-                        MediaDatabase.Playlists.Add (playlist);
-                        foreach (int track_id in ServiceManager.DbConnection.QueryEnumerable<int> (String.Format (
-                            "SELECT CoreTracks.TrackID FROM {0} WHERE {1}",
-                            from.DatabaseTrackModel.ConditionFromFragment, from.DatabaseTrackModel.Condition)))
-                        {
-                            if (tracks_map.ContainsKey (track_id)) {
-                                playlist.Tracks.Add (tracks_map[track_id].IpodTrack);
-                            }
+        void SyncRemovalOfInvalidTracks ()
+        {
+            while (invalid_tracks_in_device.Count > 0) {
+                try {
+                    DeleteTrack (invalid_tracks_in_device.Dequeue (), false);
+                } catch (Exception e) {
+                    Log.Exception ("Cannot remove invalid track from iPod", e);
+                }
+            }
+        }
+
+        void SyncTracksToPlaylists ()
+        {
+            if (!SupportsPlaylists) {
+                return;
+            }
+
+            // Remove playlists on the device
+            var device_playlists = new List<GPod.Playlist> (MediaDatabase.Playlists);
+            foreach (var playlist in device_playlists) {
+                if (!playlist.IsMaster && !playlist.IsPodcast) {
+                    MediaDatabase.Playlists.Remove (playlist);
+                }
+            }
+
+            // Add playlists from Banshee to the device
+            foreach (Source child in Children) {
+                PlaylistSource from = child as PlaylistSource;
+                if (from != null && from.Count > 0) {
+                    var playlist = new GPod.Playlist (from.Name);
+                    MediaDatabase.Playlists.Add (playlist);
+                    foreach (int track_id in ServiceManager.DbConnection.QueryEnumerable<int> (String.Format (
+                        "SELECT CoreTracks.TrackID FROM {0} WHERE {1}",
+                        from.DatabaseTrackModel.ConditionFromFragment, from.DatabaseTrackModel.Condition)))
+                    {
+                        if (tracks_map.ContainsKey (track_id)) {
+                            playlist.Tracks.Add (tracks_map[track_id].IpodTrack);
                         }
                     }
                 }
             }
+        }
 
+        void SyncDatabase (UserJob progressUpdater)
+        {
             try {
-                message = Catalog.GetString ("Writing media database");
+                string message = Catalog.GetString ("Writing media database");
                 UpdateProgress (progressUpdater, message, 1, 1);
 
                 lock (write_mutex) {
@@ -672,10 +714,6 @@ namespace Banshee.Dap.AppleDevice
             } catch (Exception e) {
                 Log.Exception ("Failed to save iPod database", e);
             }
-            MediaDatabase.StopSync ();
-            progressUpdater.Finish ();
-
-            Hyena.Log.Debug ("Ending AppleDevice sync thread cycle");
         }
 
         public bool SyncNeeded {
